@@ -1,13 +1,8 @@
 #include <assert.h>
 
-#if TARGET_OS_MAC
-// Mac OS X com_err files do not include com_err for you
-#include <Kerberos/com_err.h>
-#endif
-
-#include "gssapiP_krb5.h"
 #include "gssapi_err_generic.h"
 #include "gssapi_err_krb5.h"
+#include "gssapiP_krb5.h"
 
 #include "gss_libinit.h"
 
@@ -21,7 +16,7 @@ OM_uint32 gssint_initialize_library (void)
 {
 	
 	if (!initialized) {
-#if !TARGET_OS_MAC || USE_HARDCODED_FALLBACK_ERROR_TABLES
+#if !USE_BUNDLE_ERROR_STRINGS
 	    add_error_table(&et_k5g_error_table);
 	    add_error_table(&et_ggss_error_table);
 #endif
@@ -38,14 +33,16 @@ OM_uint32 gssint_initialize_library (void)
 
 void gssint_cleanup_library (void)
 {
-	OM_uint32 maj_stat, min_stat;
+	OM_uint32 min_stat;
 
 	assert (initialized);
 	
-	maj_stat = kg_release_defcred (&min_stat);
+	(void) kg_release_defcred (&min_stat);
 	
-    remove_error_table(&et_k5g_error_table);
-    remove_error_table(&et_ggss_error_table);
+#if !USE_BUNDLE_ERROR_STRINGS
+	remove_error_table(&et_k5g_error_table);
+	remove_error_table(&et_ggss_error_table);
+#endif
 	
 	initialized = 0;
 }
