@@ -72,13 +72,11 @@
 
 #include "k5-int.h"
 #include "gssapiP_krb5.h"
+#ifdef HAVE_MEMORY_H
 #include <memory.h>
+#endif
 #include <assert.h>
 
-/*
- * $Id: accept_sec_context.c,v 1.51.2.5.2.5 2002/04/17 00:07:17 dalmeida Exp $
- */
-
 /* Decode, decrypt and store the forwarded creds in the local ccache. */
 static krb5_error_code
 rd_and_store_for_creds(context, auth_context, inbuf, out_cred)
@@ -94,34 +92,37 @@ rd_and_store_for_creds(context, auth_context, inbuf, out_cred)
     krb5_auth_context new_auth_ctx = NULL;
 	krb5_int32 flags_org;
 
-	if (retval = krb5_auth_con_getflags(context, auth_context, &flags_org))
+	if ((retval = krb5_auth_con_getflags(context, auth_context, &flags_org)))
 		return retval;
-	krb5_auth_con_setflags(context, auth_context, 0);
+	krb5_auth_con_setflags(context, auth_context,
+			       0);
 
-	/* By the time krb5_rd_cred is called here (after krb5_rd_req has been */
-	/* called in krb5_gss_accept_sec_context), the "keyblock" field of */
-	/* auth_context contains a pointer to the session key, and the */
-	/* "remote_subkey" field might contain a session subkey.  Either of */
-	/* these (the "remote_subkey" if it isn't NULL, otherwise the */
-	/* "keyblock") might have been used to encrypt the encrypted part of */
-	/* the KRB_CRED message that contains the forwarded credentials.  (The */
-	/* Java Crypto and Security Implementation from the DSTC in Australia */
-	/* always uses the session key.  But apparently it never negotiates a */
-	/* subkey, so this code works fine against a JCSI client.)  Up to the */
-	/* present, though, GSSAPI clients linked against the MIT code (which */
-	/* is almost all GSSAPI clients) don't encrypt the KRB_CRED message at */
-	/* all -- at this level.  So if the first call to krb5_rd_cred fails, */
-	/* we should call it a second time with another auth context freshly */
-	/* created by krb5_auth_con_init.  All of its keyblock fields will be */
-	/* NULL, so krb5_rd_cred will assume that the KRB_CRED message is */
-	/* unencrypted.  (The MIT code doesn't actually send the KRB_CRED */
-	/* message in the clear -- the "authenticator" whose "checksum" ends up */
-	/* containing the KRB_CRED message does get encrypted.) */
+	/*
+	 * By the time krb5_rd_cred is called here (after krb5_rd_req has been
+	 * called in krb5_gss_accept_sec_context), the "keyblock" field of
+	 * auth_context contains a pointer to the session key, and the
+	 * "recv_subkey" field might contain a session subkey.  Either of
+	 * these (the "recv_subkey" if it isn't NULL, otherwise the
+	 * "keyblock") might have been used to encrypt the encrypted part of
+	 * the KRB_CRED message that contains the forwarded credentials.  (The
+	 * Java Crypto and Security Implementation from the DSTC in Australia
+	 * always uses the session key.  But apparently it never negotiates a
+	 * subkey, so this code works fine against a JCSI client.)  Up to the
+	 * present, though, GSSAPI clients linked against the MIT code (which
+	 * is almost all GSSAPI clients) don't encrypt the KRB_CRED message at
+	 * all -- at this level.  So if the first call to krb5_rd_cred fails,
+	 * we should call it a second time with another auth context freshly
+	 * created by krb5_auth_con_init.  All of its keyblock fields will be
+	 * NULL, so krb5_rd_cred will assume that the KRB_CRED message is
+	 * unencrypted.  (The MIT code doesn't actually send the KRB_CRED
+	 * message in the clear -- the "authenticator" whose "checksum" ends up
+	 * containing the KRB_CRED message does get encrypted.)
+	 */
 	if (krb5_rd_cred(context, auth_context, inbuf, &creds, NULL)) {
-		if (retval = krb5_auth_con_init(context, &new_auth_ctx))
+		if ((retval = krb5_auth_con_init(context, &new_auth_ctx)))
 			goto cleanup;
 		krb5_auth_con_setflags(context, new_auth_ctx, 0);
-		if (retval = krb5_rd_cred(context, new_auth_ctx, inbuf, &creds, NULL))
+		if ((retval = krb5_rd_cred(context, new_auth_ctx, inbuf, &creds, NULL)))
 			goto cleanup;
 		}
 
@@ -218,7 +219,7 @@ krb5_gss_accept_sec_context(minor_status, context_handle,
    size_t md5len;
    int bigend;
    krb5_gss_cred_id_t cred = 0;
-   krb5_data ap_rep, ap_req, mic;
+   krb5_data ap_rep, ap_req;
    int i;
    krb5_error_code code;
    krb5_address addr, *paddr;
@@ -233,14 +234,13 @@ krb5_gss_accept_sec_context(minor_status, context_handle,
    krb5_auth_context auth_context = NULL;
    krb5_ticket * ticket = NULL;
    int option_id;
-   krb5_data option, cksumdata;
+   krb5_data option;
    const gss_OID_desc *mech_used = NULL;
    OM_uint32 major_status = GSS_S_FAILURE;
    krb5_error krb_error_data;
    krb5_data scratch;
    gss_cred_id_t cred_handle = NULL;
    krb5_gss_cred_id_t deleg_cred = NULL;
-   krb5_cksumtype *ctypes = 0;
 
    if (GSS_ERROR(kg_get_context(minor_status, &context)))
       return(GSS_S_FAILURE);
@@ -253,10 +253,8 @@ krb5_gss_accept_sec_context(minor_status, context_handle,
    output_token->value = NULL;
    token.value = 0;
    reqcksum.contents = 0;
-   mic.data = 0;
    ap_req.data = 0;
    ap_rep.data = 0;
-   cksumdata.data = 0;
    
    if (mech_type)
       *mech_type = GSS_C_NULL_OID;
@@ -277,20 +275,24 @@ krb5_gss_accept_sec_context(minor_status, context_handle,
 
    /* handle default cred handle */
    if (verifier_cred_handle == GSS_C_NO_CREDENTIAL) {
-       major_status = krb5_gss_acquire_cred(&code, GSS_C_NO_NAME,
+       major_status = krb5_gss_acquire_cred(minor_status, GSS_C_NO_NAME,
 					    GSS_C_INDEFINITE, GSS_C_NO_OID_SET,
 					    GSS_C_ACCEPT, &cred_handle,
 					    NULL, NULL);
-       if (major_status != GSS_S_COMPLETE)
+       if (major_status != GSS_S_COMPLETE) {
+	   code = *minor_status;
 	   goto fail;
+       }
    } else {
+       major_status = krb5_gss_validate_cred(minor_status,
+					     verifier_cred_handle);
+       if (GSS_ERROR(major_status)) {
+	   code = *minor_status;
+	   goto fail;
+       }
        cred_handle = verifier_cred_handle;
    }
 
-   major_status = krb5_gss_validate_cred(&code, verifier_cred_handle);
-   if (GSS_ERROR(major_status))
-       goto fail;
-
    cred = (krb5_gss_cred_id_t) cred_handle;
 
    /* make sure the supplied credentials are valid for accept */
@@ -356,9 +358,13 @@ krb5_gss_accept_sec_context(minor_status, context_handle,
        major_status = GSS_S_FAILURE;
        goto fail;
    }
-   if ((code = krb5_auth_con_setrcache(context, auth_context, cred->rcache))) {
-       major_status = GSS_S_FAILURE;
-       goto fail;
+   krb5_auth_con_setflags(context, auth_context,
+			  KRB5_AUTH_CONTEXT_DO_SEQUENCE);
+   if (cred->rcache) {
+       if ((code = krb5_auth_con_setrcache(context, auth_context, cred->rcache))) {
+	   major_status = GSS_S_FAILURE;
+	   goto fail;
+       }
    }
    if ((code = krb5_auth_con_setaddrs(context, auth_context, NULL, paddr))) {
        major_status = GSS_S_FAILURE;
@@ -388,8 +394,8 @@ krb5_gss_accept_sec_context(minor_status, context_handle,
        /* gss krb5 v1 */
 
        /* stash this now, for later. */
-       if (code = krb5_c_checksum_length(context, CKSUMTYPE_RSA_MD5,
-					 &md5len)) {
+       code = krb5_c_checksum_length(context, CKSUMTYPE_RSA_MD5, &md5len);
+       if (code) {
 	   major_status = GSS_S_FAILURE;
 	   goto fail;
        }
@@ -440,22 +446,51 @@ krb5_gss_accept_sec_context(minor_status, context_handle,
        /* at this point, bigend is set according to the initiator's
 	  byte order */
 
-       if ((code = kg_checksum_channel_bindings(context, input_chan_bindings,
+
+       /* 
+          The following section of code attempts to implement the
+          optional channel binding facility as described in RFC2743.
+
+          Since this facility is optional channel binding may or may
+          not have been provided by either the client or the server.
+
+          If the server has specified input_chan_bindings equal to
+          GSS_C_NO_CHANNEL_BINDINGS then we skip the check.  If
+          the server does provide channel bindings then we compute
+          a checksum and compare against those provided by the
+          client.  If the check fails we test the clients checksum
+          to see whether the client specified GSS_C_NO_CHANNEL_BINDINGS.
+          If either test succeeds we continue without error.
+       */
+
+       if ((code = kg_checksum_channel_bindings(context, 
+						input_chan_bindings,
 						&reqcksum, bigend))) {
-	   major_status = GSS_S_BAD_BINDINGS;
-	   goto fail;
+	 major_status = GSS_S_BAD_BINDINGS;
+	 goto fail;
        }
 
-		TREAD_STR(ptr, ptr2, reqcksum.length);
-		/* If server has chosen not to use channel bindings, ignore */
-		/* any channel bindings sent by the client. */
-		if (input_chan_bindings != GSS_C_NO_CHANNEL_BINDINGS) {
-			if (memcmp(ptr2, reqcksum.contents, reqcksum.length) != 0) {
-				code = 0;
-				major_status = GSS_S_BAD_BINDINGS;
-				goto fail;
-				}
-			}
+       /* Always read the clients bindings - eventhough we might ignore them */
+       TREAD_STR(ptr, ptr2, reqcksum.length);
+
+       if (input_chan_bindings != GSS_C_NO_CHANNEL_BINDINGS ) {
+           if (memcmp(ptr2, reqcksum.contents, reqcksum.length) != 0) {
+               xfree(reqcksum.contents);
+               reqcksum.contents = 0;
+               if ((code = kg_checksum_channel_bindings(context, 
+                                                  GSS_C_NO_CHANNEL_BINDINGS,
+                                                  &reqcksum, bigend))) {
+                   major_status = GSS_S_BAD_BINDINGS;
+                   goto fail;
+               }
+               if (memcmp(ptr2, reqcksum.contents, reqcksum.length) != 0) {
+                   code = 0;
+                   major_status = GSS_S_BAD_BINDINGS;
+                   goto fail;
+               }
+           }
+           
+       }
 
        xfree(reqcksum.contents);
        reqcksum.contents = 0;
@@ -490,7 +525,7 @@ krb5_gss_accept_sec_context(minor_status, context_handle,
 	       }
 
 	       TREAD_STR(ptr, ptr2, bigend);
-	       option.data = (char FAR *) ptr2;
+	       option.data = (char *) ptr2;
 
 	       i -= option.length;
 
@@ -500,9 +535,10 @@ krb5_gss_accept_sec_context(minor_status, context_handle,
 
 		   /* store the delegated credential */
 
-		   if (code = rd_and_store_for_creds(context, auth_context, &option,
-						     (delegated_cred_handle) ?
-						     &deleg_cred : NULL)) {
+		   code = rd_and_store_for_creds(context, auth_context, &option,
+						 (delegated_cred_handle) ?
+						 &deleg_cred : NULL);
+		   if (code) {
 		       major_status = GSS_S_FAILURE;
 		       goto fail;
 		   }
@@ -546,7 +582,7 @@ krb5_gss_accept_sec_context(minor_status, context_handle,
        goto fail;
    }
 
-   if ((code = krb5_copy_principal(context, cred->princ, &ctx->here))) {
+   if ((code = krb5_copy_principal(context, ticket->server, &ctx->here))) {
        major_status = GSS_S_FAILURE;
        goto fail;
    }
@@ -556,8 +592,8 @@ krb5_gss_accept_sec_context(minor_status, context_handle,
        goto fail;
    }
 
-   if ((code = krb5_auth_con_getremotesubkey(context, auth_context,
-					     &ctx->subkey))) { 
+   if ((code = krb5_auth_con_getrecvsubkey(context, auth_context,
+					   &ctx->subkey))) { 
        major_status = GSS_S_FAILURE;      
        goto fail;
    }
@@ -625,6 +661,20 @@ krb5_gss_accept_sec_context(minor_status, context_handle,
        }
 
        break;
+	  case ENCTYPE_ARCFOUR_HMAC:
+	    ctx->signalg = SGN_ALG_HMAC_MD5 ;
+	    ctx->cksum_size = 8;
+	    ctx->sealalg = SEAL_ALG_MICROSOFT_RC4 ;
+
+	      code = krb5_copy_keyblock (context, ctx->subkey, &ctx->enc);
+	      if (code)
+		  goto fail;
+	      code = krb5_copy_keyblock (context, ctx->subkey, &ctx->seq);
+	      if (code) {
+		  krb5_free_keyblock (context, ctx->enc);
+		  goto fail;
+	      }
+	      break;	    
 
    default:
        code = KRB5_BAD_ENCTYPE;
@@ -659,7 +709,7 @@ krb5_gss_accept_sec_context(minor_status, context_handle,
    /* generate an AP_REP if necessary */
 
    if (ctx->gss_flags & GSS_C_MUTUAL_FLAG) {
-       unsigned char * ptr;
+       unsigned char * ptr3;
        if ((code = krb5_mk_rep(context, auth_context, &ap_rep))) {
 	   major_status = GSS_S_FAILURE;
 	   goto fail;
@@ -669,6 +719,7 @@ krb5_gss_accept_sec_context(minor_status, context_handle,
 				       &ctx->seq_send);
 
        /* the reply token hasn't been sent yet, but that's ok. */
+       ctx->gss_flags |= GSS_C_PROT_READY_FLAG;
        ctx->established = 1;
 
        token.length = g_token_size((gss_OID) mech_used, ap_rep.length);
@@ -679,11 +730,11 @@ krb5_gss_accept_sec_context(minor_status, context_handle,
 	   code = ENOMEM;
 	   goto fail;
        }
-       ptr = token.value;
+       ptr3 = token.value;
        g_make_token_header((gss_OID) mech_used, ap_rep.length,
-			   &ptr, KG_TOK_CTX_AP_REP);
+			   &ptr3, KG_TOK_CTX_AP_REP);
 
-       TWRITE_STR(ptr, ap_rep.data, ap_rep.length);
+       TWRITE_STR(ptr3, ap_rep.data, ap_rep.length);
 
        ctx->established = 1;
 
@@ -741,8 +792,6 @@ krb5_gss_accept_sec_context(minor_status, context_handle,
    major_status = GSS_S_COMPLETE;
 
  fail:
-   if (ctypes)
-       free(ctypes);
    if (authdat)
        krb5_free_authenticator(context, authdat);
    /* The ctx structure has the handle of the auth_context */
@@ -753,11 +802,7 @@ krb5_gss_accept_sec_context(minor_status, context_handle,
    if (reqcksum.contents)
        xfree(reqcksum.contents);
    if (ap_rep.data)
-       xfree(ap_rep.data);
-   if (mic.data)
-       xfree(mic.data);
-   if (cksumdata.data)
-       xfree(cksumdata.data);
+       krb5_free_data_contents(context, &ap_rep);
 
    if (!GSS_ERROR(major_status))
        return(major_status);
@@ -800,7 +845,8 @@ krb5_gss_accept_sec_context(minor_status, context_handle,
    }
 
    if (cred && (gss_flags & GSS_C_MUTUAL_FLAG)) {
-       int tmsglen, toktype;
+       unsigned int tmsglen;
+       int toktype;
 
        /*
 	* The client is expecting a response, so we can send an
@@ -833,12 +879,12 @@ krb5_gss_accept_sec_context(minor_status, context_handle,
        g_make_token_header((gss_OID) mech_used, tmsglen, &ptr, toktype);
 
        TWRITE_STR(ptr, scratch.data, scratch.length);
-       xfree(scratch.data);
+       krb5_free_data_contents(context, &scratch);
 
        *output_token = token;
    }
    if (!verifier_cred_handle && cred_handle) {
-	   krb5_gss_release_cred(&code, cred_handle);
+	   krb5_gss_release_cred(minor_status, cred_handle);
    }
    return (major_status);
 }
