@@ -36,9 +36,7 @@
  * routine
  */
 krb5_error_code 
-krb5_set_real_time(context, seconds, microseconds)
-    krb5_context context;
-    krb5_int32 seconds, microseconds;
+krb5_set_real_time(krb5_context context, krb5_int32 seconds, krb5_int32 microseconds)
 {
     krb5_os_context os_ctx = context->os_context;
     krb5_int32 sec, usec;
@@ -60,9 +58,7 @@ krb5_set_real_time(context, seconds, microseconds)
  * is useful for running the krb5 routines through test suites
  */
 krb5_error_code 
-krb5_set_debugging_time(context, seconds, microseconds)
-    krb5_context context;
-    krb5_int32 seconds, microseconds;
+krb5_set_debugging_time(krb5_context context, krb5_int32 seconds, krb5_int32 microseconds)
 {
     krb5_os_context os_ctx = context->os_context;
 
@@ -78,8 +74,7 @@ krb5_set_debugging_time(context, seconds, microseconds)
  * routines return the "natural" time.
  */
 krb5_error_code 
-krb5_use_natural_time(context)
-    krb5_context context;
+krb5_use_natural_time(krb5_context context)
 {
     krb5_os_context os_ctx = context->os_context;
 
@@ -92,9 +87,7 @@ krb5_use_natural_time(context)
  * This routine returns the current time offsets in use.
  */
 krb5_error_code KRB5_CALLCONV
-krb5_get_time_offsets(context, seconds, microseconds)
-    krb5_context context;
-    krb5_int32 *seconds, *microseconds;
+krb5_get_time_offsets(krb5_context context, krb5_int32 *seconds, krb5_int32 *microseconds)
 {
     krb5_os_context os_ctx = context->os_context;
 
@@ -110,9 +103,7 @@ krb5_get_time_offsets(context, seconds, microseconds)
  * This routine sets the time offsets directly.
  */
 krb5_error_code 
-krb5_set_time_offsets(context, seconds, microseconds)
-    krb5_context context;
-    krb5_int32 seconds, microseconds;
+krb5_set_time_offsets(krb5_context context, krb5_int32 seconds, krb5_int32 microseconds)
 {
     krb5_os_context os_ctx = context->os_context;
 
